@@ -4,7 +4,8 @@ import selectors
 import sys
 
 
-def asyncio_setup():
+def asyncio_setup() -> None:  # pragma: no cover
+    loop: asyncio.AbstractEventLoop
     if (
         sys.version_info.major >= 3
         and sys.version_info.minor >= 8
